package net.minecraft.client.sound;

import net.minecraft.game.MathHelper;
import net.minecraft.game.entity.EntityLiving;
import java.io.File;
import paulscode.sound.codecs.CodecWav;
import paulscode.sound.codecs.CodecJOrbis;
import paulscode.sound.SoundSystemConfig;
import paulscode.sound.libraries.LibraryLWJGLOpenAL;
import net.minecraft.client.GameSettings;
import paulscode.sound.SoundSystem;

public final class SoundManager {
    private SoundSystem sndSystem;
    private SoundPool soundPoolSounds;
    private SoundPool soundPoolMusic;
    private int latestSoundID;
    private GameSettings options;
    private boolean loaded;
    
    public SoundManager() {
        this.soundPoolSounds = new SoundPool();
        this.soundPoolMusic = new SoundPool();
        this.latestSoundID = 0;
        this.loaded = false;
    }
    
    public final void loadSoundSettings(final GameSettings gameSettings) {
        this.options = gameSettings;
        if (!this.loaded && (gameSettings.sound || gameSettings.music)) {
            this.tryToSetLibraryAndCodecs();
        }
    }
    
    private void tryToSetLibraryAndCodecs() {
        try {
            final boolean sound = this.options.sound;
            final boolean music = this.options.music;
            this.options.sound = false;
            this.options.music = false;
            this.options.saveOptions();
            SoundSystemConfig.addLibrary(LibraryLWJGLOpenAL.class);
            SoundSystemConfig.setCodec("ogg", (Class)CodecJOrbis.class);
            SoundSystemConfig.setCodec("wav", (Class)CodecWav.class);
            this.sndSystem = new SoundSystem();
            this.options.sound = sound;
            this.options.music = music;
            this.options.saveOptions();
        }
        catch (Throwable t) {
            System.err.println("error linking with the LibraryJavaSound plug-in");
        }
        this.loaded = true;
    }
    
    public final void onSoundOptionsChanged() {
        if (!this.loaded && (this.options.sound || this.options.music)) {
            this.tryToSetLibraryAndCodecs();
        }
        if (!this.options.music) {
            this.sndSystem.stop("BgMusic");
        }
    }
    
    public final void closeMinecraft() {
        if (this.loaded) {
            this.sndSystem.cleanup();
        }
    }
    
    public final void addSound(final String string, final File file) {
        this.soundPoolSounds.addSound(string, file);
    }
    
    public final void addMusic(final String string, final File file) {
        this.soundPoolMusic.addSound(string, file);
    }
    
    public final void playRandomMusicIfReady(final double double1, final double double2, final double double3) {
        if (!this.loaded || !this.options.music) {
            return;
        }
        if (!this.sndSystem.playing("BgMusic")) {
            final SoundPoolEntry randomSoundFromSoundPool = this.soundPoolMusic.getRandomSoundFromSoundPool("calm");
            this.sndSystem.newStreamingSource(true, "BgMusic", randomSoundFromSoundPool.soundUrl, randomSoundFromSoundPool.soundName, false, (float)double1, (float)double2, (float)double3, 2, 32.0f);
            this.sndSystem.play("BgMusic");
        }
    }
    
    public final void setListener(final EntityLiving entityLiving, double double2) {
        if (!this.loaded || !this.options.sound) {
            return;
        }
        if (entityLiving == null) {
            return;
        }
        double sin = entityLiving.prevRotationPitch + (entityLiving.rotationPitch - entityLiving.prevRotationPitch) * double2;
        double sin2 = entityLiving.prevRotationYaw + (entityLiving.rotationYaw - entityLiving.prevRotationYaw) * double2;
        final double double3 = entityLiving.prevPosX + (entityLiving.posX - entityLiving.prevPosX) * double2;
        final double double4 = entityLiving.prevPosY + (entityLiving.posY - entityLiving.prevPosY) * double2;
        final double double5 = entityLiving.prevPosZ + (entityLiving.posZ - entityLiving.prevPosZ) * double2;
        double2 = MathHelper.cos(-sin2 * 0.017453292f - 3.1415927f);
        sin2 = MathHelper.sin(-sin2 * 0.017453292f - 3.1415927f);
        final double cos = MathHelper.cos(-sin * 0.017453292f);
        sin = MathHelper.sin(-sin * 0.017453292f);
        final double double6 = -sin2 * cos;
        final double double7 = -double2 * cos;
        sin2 = -sin2 * sin;
        double2 = -double2 * sin;
        this.sndSystem.setListenerPosition((float)double3, (float)double4, (float)double5);
        this.sndSystem.setListenerOrientation((float)double6, (float)sin, (float)double7, (float)sin2, (float)cos, (float)double2);
    }
    
    public final void playSound(final String string, final double double2, final double double3, final double double4, double double5, final double double6) {
        if (!this.loaded || !this.options.sound) {
            return;
        }
        final SoundPoolEntry randomSoundFromSoundPool;
        if ((randomSoundFromSoundPool = this.soundPoolSounds.getRandomSoundFromSoundPool(string)) != null && double5 > 0.0f) {
            this.latestSoundID = (this.latestSoundID + 1) % 256;
            final String string2 = new StringBuilder().append("sound_").append(this.latestSoundID).toString();
            double double7 = 16.0f;
            if (double5 > 1.0f) {
                double7 = 16.0f * double5;
            }
            this.sndSystem.newSource(double5 > 1.0f, string2, randomSoundFromSoundPool.soundUrl, randomSoundFromSoundPool.soundName, false, (float)double2, (float)double3, (float)double4, 2, (float)double7);
            this.sndSystem.setPitch(string2, (float)double6);
            if (double5 > 1.0f) {
                double5 = 1.0f;
            }
            this.sndSystem.setVolume(string2, (float)double5);
            this.sndSystem.play(string2);
        }
    }
    
    public final void playSoundFX(final String string, final double double2, final double double3) {
        if (!this.loaded || !this.options.sound) {
            return;
        }
        final SoundPoolEntry randomSoundFromSoundPool;
        if ((randomSoundFromSoundPool = this.soundPoolSounds.getRandomSoundFromSoundPool(string)) != null) {
            this.latestSoundID = (this.latestSoundID + 1) % 256;
            final String string2 = new StringBuilder().append("sound_").append(this.latestSoundID).toString();
            this.sndSystem.newSource(false, string2, randomSoundFromSoundPool.soundUrl, randomSoundFromSoundPool.soundName, false, 0.0f, 0.0f, 0.0f, 0, 0.0f);
            this.sndSystem.setPitch(string2, 1.0f);
            this.sndSystem.setVolume(string2, 0.25f);
            this.sndSystem.play(string2);
        }
    }
}
